import { RegionInfo, RegionInfoProvider } from "@aws-sdk/types";

// Partition default templates
const AWS_TEMPLATE = "organizations.{region}.amazonaws.com";
const AWS_CN_TEMPLATE = "organizations.{region}.amazonaws.com.cn";
const AWS_ISO_TEMPLATE = "organizations.{region}.c2s.ic.gov";
const AWS_ISO_B_TEMPLATE = "organizations.{region}.sc2s.sgov.gov";
const AWS_US_GOV_TEMPLATE = "organizations.{region}.amazonaws.com";

// Partition regions
const AWS_REGIONS = new Set([
  "af-south-1",
  "ap-east-1",
  "ap-northeast-1",
  "ap-northeast-2",
  "ap-northeast-3",
  "ap-south-1",
  "ap-southeast-1",
  "ap-southeast-2",
  "ca-central-1",
  "eu-central-1",
  "eu-north-1",
  "eu-south-1",
  "eu-west-1",
  "eu-west-2",
  "eu-west-3",
  "me-south-1",
  "sa-east-1",
  "us-east-1",
  "us-east-2",
  "us-west-1",
  "us-west-2",
]);
const AWS_CN_REGIONS = new Set(["cn-north-1", "cn-northwest-1"]);
const AWS_ISO_REGIONS = new Set(["us-iso-east-1"]);
const AWS_ISO_B_REGIONS = new Set(["us-isob-east-1"]);
const AWS_US_GOV_REGIONS = new Set(["us-gov-east-1", "us-gov-west-1"]);

export const defaultRegionInfoProvider: RegionInfoProvider = (region: string, options?: any) => {
  let regionInfo: RegionInfo | undefined = undefined;
  switch (region) {
    // First, try to match exact region names.
    case "aws-cn-global":
      regionInfo = {
        hostname: "organizations.cn-northwest-1.amazonaws.com.cn",
        partition: "aws-cn",
        signingRegion: "cn-northwest-1",
      };
      break;
    case "aws-global":
      regionInfo = {
        hostname: "organizations.us-east-1.amazonaws.com",
        partition: "aws",
        signingRegion: "us-east-1",
      };
      break;
    case "aws-us-gov-global":
      regionInfo = {
        hostname: "organizations.us-gov-west-1.amazonaws.com",
        partition: "aws-us-gov",
        signingRegion: "us-gov-west-1",
      };
      break;
    case "fips-aws-global":
      regionInfo = {
        hostname: "organizations-fips.us-east-1.amazonaws.com",
        partition: "aws",
        signingRegion: "us-east-1",
      };
      break;
    case "fips-aws-us-gov-global":
      regionInfo = {
        hostname: "organizations.us-gov-west-1.amazonaws.com",
        partition: "aws-us-gov",
        signingRegion: "us-gov-west-1",
      };
      break;
    // Next, try to match partition endpoints.
    default:
      if (AWS_REGIONS.has(region)) {
        return defaultRegionInfoProvider("aws-global");
      }
      if (AWS_CN_REGIONS.has(region)) {
        return defaultRegionInfoProvider("aws-cn-global");
      }
      if (AWS_ISO_REGIONS.has(region)) {
        regionInfo = {
          hostname: AWS_ISO_TEMPLATE.replace("{region}", region),
          partition: "aws-iso",
        };
      }
      if (AWS_ISO_B_REGIONS.has(region)) {
        regionInfo = {
          hostname: AWS_ISO_B_TEMPLATE.replace("{region}", region),
          partition: "aws-iso-b",
        };
      }
      if (AWS_US_GOV_REGIONS.has(region)) {
        return defaultRegionInfoProvider("aws-us-gov-global");
      }
      // Finally, assume it's an AWS partition endpoint.
      if (regionInfo === undefined) {
        return defaultRegionInfoProvider("aws-global");
      }
  }
  return Promise.resolve({ signingService: "organizations", ...regionInfo });
};
